/**
 * Author: Michael Feilen
 *
 * Date created: 29.05.2009
 * Last changed: 05.06.2009 (mfeilen)
 *
 * Short description: THIS IS THE DEVICE INTERFACE FOR SPARK
 *                    PLEASE USE THIS INTERFACE FOR YOUR PCM INPUT OR OUTPUT LINE IMPLEMENTATION.
 *
 * Description: The functions can be used to interface a PCM device with
 *              one or more input or output lines. Each line is allowed to have its
 *              own specific sample rate, number of channels, number of
 *              bytes per sample and name.
 *              To use the output device, an output device handle must be created. 
 *              In order to write data to a line of the output device, 
 *              the specific line must be opened with a certain lineUID. 
 *              The line ID is defined as an integer number
 *              between 0 and numLines-1.
 *              After the line was started, the read/write function can be 
 *              subsequently accessed with the corresponding lineUID.
 *              To stop the output activity, the stop function is called.
 *              Note that as long the line was not closed it is not allowed
 *              to use the same line from within another thread.
 *
 *              --- DO NOT EDIT THIS FILE ---
 */

#ifndef NATIVEPCMOUTDEV_H
#define NATIVEPCMOUTDEV_H
#include "types.h"


/**
 * Opens the line with lineUID of the output device.
 */
DEV_ERR_CODE Native_Line_Open(uint32 lineUID);

/**
 * Query if the device is opened
 */
DEV_ERR_CODE Native_Line_IsOpen(uint32 lineUID, bool8 *isOpen);

/**
 * Query if the device is started
 */
DEV_ERR_CODE Native_Line_IsStarted(uint32 lineUID, bool8 *isStarted);

/**
 * Closes the line with lineUID of the output device. 
 */
DEV_ERR_CODE Native_Line_Close(uint32 lineUID);

/**
 * Starts the line with lineUID of the output device.
 */
DEV_ERR_CODE Native_Line_Start(uint32 lineUID);

/**
 * Stops the line with lineUID of the output device. An output device once stopped
 * might be started again.
 */
DEV_ERR_CODE Native_Line_Stop(uint32 lineUID);

/**
 * This function can be used to get the number of 
 * writable bytes in the output buffer of
 * the line with lineUID.
 * If the output device buffer is full, numWritable is equal to zero.
 */
DEV_ERR_CODE Native_Line_GetNumBytesReadWrite(uint32 lineUID, uint32 *numBytesWriteRead);

/**
 * Write the samples at *pBytes to the output device line
 * with lineUID. The number of bytes written to the
 * output device line is returned by numWritten.
 * The sample format is as follows:
 * 8 bit  (1 byte per sample)
 * 16 bit (2 bytes per sample)
 * 24 bit (3 bytes per sample)
 * 32 bit (4 bytes per sample)
 * If there is not enough buffer space available to write to the 
 * output buffer, the write method is allowed to block.
 */
DEV_ERR_CODE Native_Line_ReadWrite(uint32 lineUID, char pBytes[], uint32 numBytes, bool8 isSigned, uint32 *numReadWritten);

/**
 * This function returns the length of 
 * the specific line output buffer in bytes.
 * NOTE: Buffer lengths < 500 ms are not recommended.
 */
DEV_ERR_CODE Native_Line_GetNumBytesBuffer(uint32 lineUID, uint32 *numBytes);

/**
 * The name of the device which is shown in the output dialog of Spark is written to name.
 * The numChars parameter indicates the number of bytes available to write to name[].
 * NOTE: This function always returns the number of characters 
 *       required to fill the msg array.
 */
DEV_ERR_CODE Native_Device_GetName(char name[], uint32 numChars, uint32 *numCharsRequired);

/**
 * The version of the device driver is written to the version array. 
 * The numChars parameter indicates the number of bytes
 * available to write to version[].
 * NOTE: This function always returns the number of characters 
 *       required to fill the version array incl. zero delimiter.
 */
DEV_ERR_CODE Native_Device_GetVersion(char version[], uint32 numChars, uint32 *numCharsRequired);

/**
 * The name of the line with lineUID is written to the name array.
 * The numChars parameter indicates the number of bytes
 * available to write to name[].
 * NOTE: This function always returns the number of characters 
 *       required to fill the name array incl. zero delimiter.
 */
DEV_ERR_CODE Native_Line_GetName(uint32 lineUID, char name[], uint32 numChars, uint32 *numCharsRequired);

/**
 * Returns the UIDs of all lines available on this device.
 */
DEV_ERR_CODE Native_Device_GetLineUIDs(uint32 uids[], uint32 len);

/**
 * Returns the number of lines available
 */
DEV_ERR_CODE Native_Device_GetNumLines(uint32 *numLines);

/**
 * The user can query the error message corresponding to a certain error code
 * by calling this function. 
 * The error message corresponing the the error code is written to the msg array.
 * The numChars parameter indicates the number of bytes
 * available to write to msg[].
 * NOTE: This function always returns the number of characters 
 *       required to fill the msg array.
 */
void Native_GetErrorMsg(DEV_ERR_CODE code, char msg[], uint32 numChars, uint32 *numCharsRequired);

/**
 * Returns true (1) if the error code in the argument indicates an error.
 * Returns false (0) otherwise.
 */
bool8 Native_IsError(DEV_ERR_CODE code);

/**
 * Returns true (1) if the error code in the argument indicates a warning.
 * Returns false (0) otherwise.
 */
bool8 Native_IsWarning(DEV_ERR_CODE code);

/**
 * Returns the samplerate of the output line with lineUID in Hz.
 * It is recommended to use samplerates in multiples of 1 kHz (1000 Hz).
 * (e.g. 93000 Hz)
 */
DEV_ERR_CODE Native_Line_GetSampleRateHz(uint32 lineUID, uint32 *sampleRateHz);

/**
 * Returns the number of bytes per sample of the output line with lineUID. (e.g. 16 bit resolution -> 2 bytes per sample)
 * It is recommended to use values between 1 and 4.
 */
DEV_ERR_CODE Native_Line_GetNumBytesPerSample(uint32 lineUID, uint32 *numBytesPerSample);

/**
 * Returns the number of channels of the output line with lineUID.
 * NOTE:
 * In case of I/Q output, the number of channels is 2 (complex signal).
 * In case of I output only, the number of channels is 1 (real signal).
 * Values other than 1 or 2 are not permitted.
 */
DEV_ERR_CODE Native_Line_GetNumChannels(uint32 lineUID, uint32 *numChannels);

/**
 * Writes the line type to *lineType. If the line is a source line, i.e. the samples
 * are read from the device, the line type is SOURCE_LINE.
 * If the line is a target line, i.e. the samples are written to the device
 * the line type is a TARGET_LINE.
 */
DEV_ERR_CODE Native_Line_GetType(uint32 lineUID, LineType *lineType);
#endif
